unit Computer;

interface
uses
  PokerPlayer, Pack;
type
  TComputerPlayer = class (TPokerPlayer)
  private
    FBluff1: Boolean;
    FBluff0: Boolean;
    FDrawn: Boolean;
    FPotAtDraw: Integer;
    RaizeCount: Integer;
  public
    procedure InitNewDeal(const APack: TPack); override;
    procedure ChangeCards;
    procedure BetOrPass(const Player: TPokerPlayer);
    procedure RaizeOrCall(const Player: TPokerPlayer);
    property PotAtDraw: Integer write FPotAtDraw;
    property Drawn: Boolean read FDrawn;
  end;

implementation
uses
  Main;

procedure TComputerPlayer.InitNewDeal(const APack: TPack);
begin
  inherited InitNewDeal(APack);
  FBluff0 := False;
  FBluff1 := False;
  FDrawn := False;
  FPotAtDraw := 0;
  RaizeCount := 0;
end;

procedure TComputerPlayer.ChangeCards;
var
  i,x: Integer;
begin
  FDrawn := True;
  RaizeCount := 0;
  case Rank of
    rkThrees: begin
                x:= random(6);
                if x = 0 then
                  case ThreesPosition of
                    4,5: DrawCard(1);
                    3: DrawCard(5);
                  end
                else
                  case ThreesPosition of
                    5: for i:=1 to 2 do
                         DrawCard(i);
                    4: begin
                         DrawCard(1);
                         DrawCard(5);
                       end;
                    3: for i:=4 to 5 do
                         DrawCard(i);
                  end;
              end;
    rkTwoPair: DrawCard(Discard);
    rkPair: begin
              if UnRanked = urkFourStraightFlush then
                DrawCard(Discard)
              else
                if FBluff1 then
                  DrawCard(1);
              if (not FBluff1) and (not FBluff0) then
                case PairPosition of
                  5: for i:=1 to 3 do
                       DrawCard(i);
                  4: begin
                       DrawCard(1);
                       DrawCard(2);
                       DrawCard(5);
                     end;
                  3: begin
                       DrawCard(1);
                       DrawCard(4);
                       DrawCard(5);
                     end;
                  2: for i:=3 to 5 do
                      DrawCard(i);
                end;
             end;
    rkFours: if FoursPosition = 5 then
                DrawCard(1)
              else
                DrawCard(5);

    rkNothing: case UnRanked of
                 urkTwoHighCards: for i:=1 to 3 do
                                    DrawCard(i);
                 urkNoHighCard, urkOneHighCard: for i:=1 to 4 do
                                                  DrawCard(i);
                 urkFourFlush..urkFourStraightFlush: DrawCard(Discard);
               end;
  end;
  SortHand(1,5);
  SetRank;
end;

procedure TComputerPlayer.BetOrPass (const Player: TPokerPlayer);
{Computer is first to bet or Player has passed}
var
  x : Integer;

  procedure Pass;
  begin
    BetValue := 0;
    Passed := True;
  end;

  procedure BetDefault;
  begin
    case Rank of
      rkNothing: if UnRanked = urkTwoHighCards then
                    BetValue := 10
                  else Pass;
      rkPair: case SubRank of                  // value of pair
                vlEight..vlJack: BetValue := 10;
                vlQueen, vlKing: BetValue := 20;
                vlAce: BetValue := 50;
                else Pass;
              end;
      rkTwoPair: case SubRank of
                   vlTwo..vlTen: BetValue := 20;
                   else BetValue := 50;
                 end;
      else BetValue := 50;
    end;
  end;

  procedure BetHigh;
  begin
    case Rank of
      rkNothing: case UnRanked of
                   urkTwoHighCards: BetValue := 20;
                   urkOneHighCard: BetValue := 10;
                   else Pass;
                 end;
      rkPair: case SubRank of
                vlTwo..vlTen: BetValue := 20;
                vlJack: BetValue := 50;
                vlQueen: BetValue := 100;
                vlKing, vlAce: BetValue := 200;
              end;
      rkTwoPair: case SubRank of
                   vlTwo..vlTen: BetValue := 50;
                   else BetValue:= 100;
                 end;
      else BetValue := 100
    end;
  end;

  procedure BetLow;
  begin
    if (SubRank >= vlJack) or (Rank >= rkTwoPair) then
      BetValue := 10
    else Pass;
  end;

  procedure SetBetLevel;
  var
    x : Integer;
  begin
    x := random(6);
    if Player.Passed then
      case x of
        0..3: BetLow;      // Bet low more frequently if user has passed
        4: BetDefault;
        5: BetHigh;
      end
    else
      case x of
        0: BetLow;
        1..4: BetDefault;
        5 : BetHigh;
      end;
  end;

  procedure BeforeDraw;
  begin
    if UnRanked = urkFourStraightFlush then
      BetValue := 50
    else
      SetBetLevel;
  end;

  procedure AfterDraw;
  begin
    if (FBluff1 or FBluff0) then
      BetValue := 200
    else
      if FPotAtDraw >= 220 then
        BetHigh
      else
        SetBetLevel;
  end;

begin  {BetOrPass}
  BetValue := 0;
  RaizeValue := 0;
  Passed := False;
  Called := False;
  Raized := False;
  if not FDrawn
  then
    BeforeDraw
  else
    AfterDraw;
  if (BetValue < SelectedValue) and (not Passed) then
  begin
    x := Random(3);
    if x = 0 then
      BetValue := SelectedValue
    else
      Pass;
  end;
end;

procedure TComputerPlayer.RaizeOrCall (const Player : TPokerPlayer);
{ Raise, Call or Fold}
  procedure Raize(AValue : Integer);
  begin
    if AValue < SelectedValue then
      AValue := SelectedValue;
    if not Player.Raized then
      BetValue := Player.BetValue + AValue
    else                                        // If raising a bet
      BetValue := Player.RaizeValue + AValue;   // If raising a raise
    RaizeValue := AValue;
    Raized := True;
    inc(RaizeCount);  // Keep count to assign number of possible re-raises before Calling
  end;

  procedure Call;
  begin
    if not Player.Raized then
      BetValue := Player.BetValue
    else
      BetValue := Player.RaizeValue;
    Called := True;
  end;

  procedure Fold;
  begin
    BetValue := 0;
    Folded := True;
  end;

  procedure BetLow;
  begin
    case Rank of
      rkNothing: case FDrawn of
                   True: if (UnRanked = urkTwoHighCards) and (Player.BetValue <= 20)
                          and (Player.NumberDrawn >= 3) then
                            Call
                         else
                           Fold;
                   False: if (UnRanked = urkTwoHighCards) and
                            (Player.BetValue <= 20) then
                            Call
                          else
                            Fold;
                 end;
      rkPair: if (SubRank >= vlNine) and (Player.BetValue <= 20)then
                Call
              else Fold;
      rkTwoPair: Call;
      else
        if RaizeCount = 0 then
          Raize(50)
        else
          Call;
    end;
  end;

  procedure BetDefault;
  begin
     case Rank of
      rkNothing: case FDrawn of
                   True: if ((UnRanked = urkTwoHighCards) or (CardValue(5) = vlAce))
                           and (Player.BetValue <= 20) and (Player.NumberDrawn >= 3)then
                             Call
                         else
                           Fold;
                   False: if ((UnRanked = urkTwoHighCards) or (CardValue(5) = vlAce))
                            and (Player.BetValue <= 20)then
                              Call
                          else
                            Fold;
                 end;
      rkPair: if (SubRank >= vlKing) and (RaizeCount = 0) then
                Raize(20)
              else
                if SubRank >= vlTen then
                  Call
                else
                  Fold;
      rkTwoPair: if Raizecount = 0 then
                   Raize(50)
                 else Call;
      else Call;
    end;
  end;

  procedure BetHigh;
  begin
    case Rank of
      rkNothing: if UnRanked >= urkOneHighCard then
                   Call
                 else Fold;
      rkPair: if (SubRank >= vlKing) and (RaizeCount = 0) then
                Raize(50)
              else Call;
      rkTwoPair: if (SubRank >= vlJack) and (RaizeCount <= 1) then
                   Raize(100)
                 else Call;
      else if Raizecount <= 3 then Raize(200)
           else Call;
    end;
  end;

  procedure SetBetLevel;
  var
    X : Integer;
  begin
    x := Random(6);
    case x of
      0    : BetLow;
      1..4 : BetDefault;
      5    : BetHigh;
    end;
  end;

  procedure BeforeDraw;
    function Bluff : Boolean;
    { Bluff on certain hands if Computer passed first(returns true) or do
      nothing (returns false)}
    begin
      if Passed then
      begin
        if (Rank = rkPair) and (SubRank = vlTen)then
          FBluff1 := True
        else
          if ((Rank = rkPair) and (SubRank = vlNine)) then
            FBluff0 := True;
        if (FBluff1 or FBluff0) then
        begin
          if RaizeCount <= 1 then  // Call if re-raised
            Raize(100)
          else
            Call;                  // Raise or call any bet
          Result := True;
        end
        else
          Result := False;
      end
      else
        Result := False;
    end;

  begin {BeforeDraw}
    if UnRanked = urkFourStraightFlush then
      if RaizeCount = 0 then
        Raize(50)
      else Call
    else
      if not Bluff then
        SetBetLevel;
  end;

  procedure AfterDraw;
  begin
    if (Rank >= rkFours) and (RaizeCount <= 10) then
      Raize(200)
    else
      if (Rank = rkFullHouse) and (RaizeCount <= 3) then
        Raize(100)
      else
        if FPotAtDraw >= 220 then
          BetHigh
        else
          SetBetLevel;
  end;

begin
  BetValue := 0;
  RaizeValue := 0;
  Passed := False;
  Called := False;
  Folded := False;
  Raized := False;
  if not FDrawn then
    BeforeDraw
  else
    AfterDraw;
  end;
end.
