unit Main;

{Don's Draw Poker
 version 1.0  5th July 2000
 Copyright (c) 2000 by Don Flockhart}
 
interface
uses
  SysUtils, Graphics, Forms, Dialogs, MMSystem, ExtCtrls, Controls, StdCtrls,
  Classes, Menus, Hemibtn, ALScrollingText, Card, Pack, PokerPlayer, Computer,
  User;

type
  TCardComponents = array[1..10] of TCard;
  TBetValueButtons = array[1..5] of THemisphereButton;
  TFormDrawPoker = class(TForm)
    Card1: TCard;
    Card2: TCard;
    Card3: TCard;
    Card4: TCard;
    Card5: TCard;
    Card6: TCard;
    Card7: TCard;
    Card8: TCard;
    Card9: TCard;
    Card10: TCard;
    DealTimer: TTimer;
    ComputerDrawTimer: TTimer;
    UserDrawTimer: TTimer;
    Scroll: TALScrollingText;
    HemBtnDeal: THemisphereButton;
    HemBtnCall: THemisphereButton;
    HemBtnPass: THemisphereButton;
    HemBtnFold: THemisphereButton;
    HemBtnDraw: THemisphereButton;
    LblHeld1: TLabel;
    LblHeld2: TLabel;
    LblHeld3: TLabel;
    LblHeld4: TLabel;
    LblHeld5: TLabel;
    HemBtn10: THemisphereButton;
    HemBtn20: THemisphereButton;
    HemBtn50: THemisphereButton;
    HemBtn100: THemisphereButton;
    HemBtn200: THemisphereButton;
    HemBtnBet: THemisphereButton;
    HemBtnRaize: THemisphereButton;
    LblComputerBet: TLabel;
    LblPot: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure HemBtnDealClick(Sender: TObject);
    procedure HemBtnFoldClick(Sender: TObject);
    procedure HemBtnDrawClick(Sender: TObject);
    procedure HemBtnCallClick(Sender: TObject);
    procedure HemBtnPassClick(Sender: TObject);
    procedure HemBtnBetClick(Sender: TObject);
    procedure HemBtn10Click(Sender: TObject);
    procedure Card1Click(Sender: TObject);
    procedure DealTimerTimer(Sender: TObject);
    procedure ComputerDrawTimerTimer(Sender: TObject);
    procedure UserDrawTimerTimer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    Pack: TPack;
    UserPlayer: TUserPlayer;
    ComputerPlayer: TComputerPlayer;
    DealNo: Integer;
    Pot: Integer;
    CardIndex: Integer;
    CardComponents: TCardComponents;
    BetValueButtons: TBetValueButtons;
    SelectedButton: THemisphereButton;
    function  ComputerBetsFirst: Boolean;
    procedure EnableButton(var AButton: THemisphereButton);
    procedure DisableButton(var AButton: THemisphereButton);
    procedure DisableAllButtons;
    procedure SetBetValueButtons;
    procedure HideBetValueButtons;
    procedure SetButtonsForBet;
    procedure SetButtonsForRaize;
    procedure SetCards(const Player: TPokerPlayer);
    procedure SetForDraw;
    procedure HideCards;
    procedure DisplayAccount;
    procedure DisplayBet;
    procedure AddBetToPot(const Player: TPokerPlayer);
    procedure ShowComputerHand;
    procedure ShowDown;
    procedure ComputerDraw;
    function  FindWinner: TPokerPlayer;
  end;
var
  FormDrawPoker: TFormDrawPoker;
  SelectedValue: Integer;

implementation

{$R *.DFM}

const
  Ante = 10;

function TFormDrawPoker.ComputerBetsFirst: Boolean;
begin
  if not Odd(DealNo) then
    Result := True
  else
    Result:= False;
end;

procedure TFormDrawPoker.EnableButton(var AButton: THemisphereButton);
begin
  AButton.Enabled := True;
  AButton.Visible := True;
end;

procedure TFormDrawPoker.DisableButton(var AButton : THemisphereButton);
begin
  AButton.Enabled := False;
  AButton.Visible := False;
end;

procedure TFormDrawPoker.DisableAllButtons;
begin
  DisableButton(HemBtnDeal);
  DisableButton(HemBtnDraw);
  DisableButton(HemBtnPass);
  DisableButton(HemBtnCall);
  DisableButton(HemBtnBet);
  DisableButton(HemBtnRaize);
  DisableButton(HemBtnFold);
end;

procedure TFormDrawPoker.SetButtonsForBet;
{User can pass or bet}
begin
    EnableButton(HemBtnPass);
    EnableButton(HemBtnBet);
end;

procedure TFormDrawPoker.SetButtonsForRaize;
{User can call, raise or fold}
begin
   EnableButton(HemBtnCall);
   EnableButton(HemBtnFold);
   EnableButton(HemBtnRaize);
end;

procedure TFormDrawPoker.SetBetValueButtons;
var
  AValue : Integer;
begin
  if ComputerPlayer.Raized then
    AValue := ComputerPlayer.RaizeValue // Minimum bet available = value of last
  else                                  // bet or raise
    AValue := ComputerPlayer.BetValue;
  case AValue of
    20 : HemBtn10Click(HemBtn20);
    50 : HemBtn10Click(HemBtn50);
   100 : HemBtn10Click(HemBtn100);
   200 : HemBtn10Click(HemBtn200);
  end;
  HideBetValueButtons;
end;

procedure TFormDrawPoker.HideBetValueButtons;
{Disable and hide buttons that show unavailable bets}
var
  i : Integer;
  Done : Boolean;
begin
  i := 1;
  Done := False;
  while (i <= 5) and (not Done) do
    if SelectedButton <> BetValueButtons[i] then
    begin
      BetValueButtons[i].Enabled := False;
      BetValueButtons[i].Visible := False;
      Inc(i);
    end else
      Done := True;
end;

procedure TFormDrawPoker.SetCards(const Player: TPokerPlayer);
{Set the card faces of the TCard components}
var
  i:Integer;
  Symbol : TShortSuit;
  Val : TCardValue;
begin
  Symbol := H;
  Val := 1;
  DealTimer.Enabled := False;
  for i:= 1 to 5 do
  begin
    case Player.SuitValue(i)  of
      stSpade: Symbol := S;
      stHeart: Symbol := H;
      stDiamond: Symbol := D;
      stClub: Symbol := C;
    end;
    case Player.CardValue(i) of
      vlTwo: Val := 2;
      vlThree: Val := 3;
      vlFour: Val := 4;
      vlFive: Val := 5;
      vlSix: Val := 6;
      vlSeven: Val := 7;
      vlEight: Val := 8;
      vlNine: Val := 9;
      vlTen: Val := 10;
      vlJack: Val := 11;
      vlQueen: Val := 12;
      vlKing: Val := 13;
      vlAce: Val := 1;
    end;
    if Player is TUserPlayer then
      CardComponents[i].SetCard(Val,Symbol)         // User cards
    else
      CardComponents[i + 5].SetCard(Val,Symbol)     // Computer cards
  end;
end;

procedure TFormDrawPoker.SetForDraw;
begin
  DisableAllButtons;
  EnableButton(HemBtnDraw);
  Card1.Cursor := crHandPoint;        // Cursor is hand while cards can be
  Card2.Cursor := crHandPoint;        // clicked to hold
  Card3.Cursor := crHandPoint;
  Card4.Cursor := crHandPoint;
  Card5.Cursor := crHandPoint;
  if not ComputerBetsFirst then
    Caption := '   Click on cards you want to keep';
end;

procedure TFormDrawPoker.DisplayAccount;
{If the user is winning the amount is displayed in green - if losing it is
 displayed in red}
begin
  if UserPlayer.Account >= 0 then
    Scroll.Font.Color := clLime
  else
    Scroll.Font.Color := clRed;
  Scroll.Text := intToStr(UserPlayer.Account);
end;

procedure TFormDrawPoker.ShowComputerHand;
{Turn the cards face up at the end of a hand}
var
  i: Integer;
begin
  for i:= 6 to 10 do
    CardComponents[i].showDeck := False;
end;

procedure TFormDrawPoker.HideCards;
var
  i : Integer;
begin
  for i := 1 to 10 do
    CardComponents[i].Hide;
end;

procedure TFormDrawPoker.DisplayBet;
var
  S : string;
begin
  if ComputerPlayer.Folded then
    S :='Fold'
  else
    if ComputerPlayer.Passed then
      S := 'Pass'
    else
      if ComputerPlayer.Called then
        S := 'Call'
      else
        if ComputerPlayer.Raized then
          S := 'Raise  ' + IntToStr(ComputerPlayer.RaizeValue)
        else
          S := 'Bet  ' + IntToStr(ComputerPlayer.BetValue);
  LblComputerBet.Caption := S;
end;

procedure TFormDrawPoker.AddBetToPot(const Player: TPokerPlayer);
begin
  Inc(Pot, Player.BetValue);
  LblPot.Caption := IntToStr(Pot);
end;

procedure TFormDrawPoker.ShowDown;
begin
  if not ComputerPlayer.Folded then
    ShowComputerHand;
  if (FindWinner = UserPlayer) or (ComputerPlayer.Folded) then
  begin
    Caption := '     YOU WIN';
    UserPlayer.Account := UserPlayer.Account + Pot;
  end else
    if FindWinner = ComputerPlayer then
      Caption := '     YOU LOSE'
    else begin
      Caption := '     EQUAL HANDS  SPLIT POT';
      UserPlayer.Account := UserPlayer.Account + Pot div 2;
    end;
  Pot := 0;
  EnableButton(HemBtnDeal);
end;

procedure TFormDrawPoker.ComputerDraw;
var
  i : Integer;
begin
  ComputerPlayer.PotAtDraw := Pot;
  ComputerPlayer.ChangeCards;
  for i:= 1 to ComputerPlayer.NumberDrawn do
  begin
    CardComponents[i+5].Hide;
  end;
  CardIndex := 6;
  ComputerDrawTimer.enabled := True;
  if ComputerBetsFirst then
    SetForDraw;
end;

function TFormDrawPoker.FindWinner: TPokerPlayer;
  procedure CompareCardByCard(i: Integer);
  {e.g if both hands have the same pair, compare the highest card, then the next
  highest and so on}
  var
    WinnerFound : Boolean;
  begin
    WinnerFound := False;
    while (not WinnerFound) and (i > 0) do
    begin
      if ComputerPlayer.CardValue(i) > UserPlayer.CardValue(i) then
      begin
        WinnerFound := True;
        Result := ComputerPlayer;
      end else
        if UserPlayer.CardValue(i) > ComputerPlayer.CardValue(i) then
        begin
          WinnerFound := True;
          Result := UserPlayer;
        end;
      Dec(i);
    end;
    if not WinnerFound then
      Result := nil;
  end;

begin
  if ComputerPlayer.Rank > UserPlayer.Rank then   // e.g.  Rank = Pair
    Result := ComputerPlayer                      // SubRank = Aces
  else                                            // If TwoPairs (Aces and Eights) -
    if UserPlayer.Rank > ComputerPlayer.Rank then // SubSubRank = Eights
      Result := UserPlayer
    else
      if ComputerPlayer.SubRank > UserPlayer.SubRank then
        Result := ComputerPlayer
      else
        if UserPlayer.SubRank > ComputerPlayer.SubRank then
          Result := UserPlayer
        else
           if ComputerPlayer.Rank = rkTwoPair then
           begin
             if ComputerPlayer.SubSubRank > UserPlayer.SubSubRank then
               Result := ComputerPlayer
             else
               if UserPlayer.SubSubRank > ComputerPlayer.SubSubRank then
                 Result := UserPlayer
               else
                 CompareCardByCard(1);
           end else
             case ComputerPlayer.Rank of
               rkNothing,rkStraight, rkFlush, rkStraightFlush : CompareCardByCard(5);
               rkPair : CompareCardByCard(3);
             end;
end;

procedure TFormDrawPoker.FormCreate(Sender: TObject);
begin
  CardComponents[1] := card1;  // Users cards 1 - 5
  CardComponents[2] := card2;
  CardComponents[3] := card3;
  CardComponents[4] := card4;
  CardComponents[5] := card5;
  CardComponents[6] := card6;  // Computers cards 6 - 10
  CardComponents[7] := card7;
  CardComponents[8] := card8;
  CardComponents[9] := card9;
  CardComponents[10] := card10;
  BetValueButtons[1] := HemBtn10;
  BetValueButtons[2] := HemBtn20;
  BetValueButtons[3] := HemBtn50;
  BetValueButtons[4] := HemBtn100;
  BetValueButtons[5] := HemBtn200;
  Scroll.Text := '0';
  DealNo := -1;                    // First deal is Number 0
  Pack := TPack.Create;
  ComputerPlayer := TComputerPlayer.Create;
  UserPlayer := TUserPlayer.Create;
  HemBtnDealClick(Self);
end;

procedure TFormDrawPoker.HemBtnDealClick(Sender: TObject);
var
  i : Integer;
begin
  HideCards;  // Clear cards off the table for new deal
  Inc(DealNo);  // Players deal in turn according to DealNo being odd or even
  Caption := 'DON''S DRAW POKER';
  LblComputerBet.Caption := '';
  DisableAllButtons;
  SelectedValue := 10;  // Minimum bet
  BetValueButtons[1].FaceColor  := clRed;
  for i := 2 to 5 do
    BetValueButtons[i].FaceColor  := clYellow;
  for i := 1 to 4 do
    begin
      BetValueButtons[i].Enabled := True;
      BetValueButtons[i].Visible := True;
    end;
  for i:= 6 to 10 do
    CardComponents[i].ShowDeck := True;  // Compuputer's cards face down
  if ComputerBetsFirst then
    CardIndex:= 6
  else
    CardIndex:= 1;
  Pack.Shuffle;
  UserPlayer.InitNewDeal(Pack);
  UserPlayer.Account := UserPlayer.Account - Ante;
  DisplayAccount;
  ComputerPlayer.InitNewDeal(pack);
  Pot := 2 * Ante;
  LblPot.Caption := IntToStr(Pot);
  SetCards (UserPlayer);
  SetCards (ComputerPlayer);
  DealTimer.Enabled := True;
end;

procedure TFormDrawPoker.HemBtnDrawClick(Sender: TObject);
var
  i : Integer;
begin
  DisableButton(HemBtnDraw);
  Caption := 'DON''S DRAW POKER';
  LblComputerBet.Caption := '';
  if not LblHeld1.Visible then
  begin
    UserPlayer.DrawCard(1);
    Card1.Hide;
  end;
  if not LblHeld2.Visible then
  begin
    UserPlayer.DrawCard(2);
    Card2.Hide;
  end;
  if not LblHeld3.Visible then
  begin
    UserPlayer.DrawCard(3);
    Card3.Hide;
  end;
  if not LblHeld4.Visible then
  begin
    UserPlayer.DrawCard(4);
    Card4.Hide;
  end;
  if not LblHeld5.Visible then
  begin
    UserPlayer.DrawCard(5);
    Card5.Hide;
  end;
  LblHeld1.Visible := False;
  LblHeld2.Visible := False;
  LblHeld3.Visible := False;
  LblHeld4.Visible := False;
  LblHeld5.Visible := False;
  CardIndex:=1;
  SetCards (UserPlayer);
  UserDrawTimer.Enabled := True;
  for i := 1 to 5 do
   CardComponents[i].Cursor := crDefault;
  if ComputerBetsFirst then
  begin
    ComputerPlayer.BetOrPass(UserPlayer);
    if ComputerPlayer.BetValue > 10 then
      SetBetValueButtons;
    DisplayBet;
    AddBetToPot (ComputerPlayer);
    if ComputerPlayer.Passed then
      SetButtonsForBet
    else
      SetButtonsForRaize;
  end else
    SetButtonsForBet;
end;

procedure TFormDrawPoker.HemBtnBetClick(Sender: TObject);
begin
  LblComputerBet.Caption := '';
  if Sender = HemBtnBet then
    UserPlayer.Bet(SelectedValue)
  else
    if Sender = HemBtnRaize then
      UserPlayer.Raize (ComputerPlayer, SelectedValue);
  DisableAllButtons;
  if UserPlayer.BetValue > 10 then
    HideBetValueButtons;
  AddBetToPot(UserPlayer);
  DisplayAccount;
  ComputerPlayer.RaizeOrCall(UserPlayer);
  if ComputerPlayer.BetValue > 10 then
    SetBetValueButtons;
  DisplayBet;
  AddBetToPot(ComputerPlayer);
  //Computer has folded, called or raised...
  if ComputerPlayer.Folded then              // Computer folded
    Showdown
  else
    if ComputerPlayer.Raized then            // Computer raised
      SetButtonsForRaize
    else                                     // Computer must have called
      if ComputerPlayer.Drawn then
        ShowDown                             // End of game if after the draw
      else
        if ComputerBetsFirst then            // Else Computer or player to draw
          ComputerDraw
        else
          SetForDraw;
end;

procedure TFormDrawPoker.HemBtnCallClick(Sender: TObject);
begin
  LblComputerBet.Caption := '';
  DisableAllButtons;
  UserPlayer.Call(ComputerPlayer);
  AddBetToPot(UserPlayer);
  DisplayAccount;
  if ComputerPlayer.Drawn then
    ShowDown                  // End of game if after the draw
  else
    if ComputerBetsFirst then
      ComputerDraw            // Else Computer or player to draw
    else
      SetForDraw;
end;

procedure TFormDrawPoker.HemBtnPassClick(Sender: TObject);
begin
  LblComputerBet.Caption := '';
  DisableAllButtons;
  UserPlayer.Pass;
  if ComputerBetsFirst then       // Both players have passed
    if ComputerPlayer.Drawn then  // End of game if after the draw
      ShowDown
    else
      ComputerDraw                // Computer to draw first
  else begin                      // Player has passed the bet to Computer
    ComputerPlayer.BetOrPass(UserPlayer);
    if ComputerPlayer.BetValue > 10 then
      SetBetValueButtons;
    DisplayBet;
    AddBetToPot (ComputerPlayer);
    if ComputerPlayer.Passed then         // Computer Passed
      if ComputerPlayer.Drawn then
        ShowDown                          // End of game if after the draw
      else
        SetForDraw                        // Player to draw first
    else
      SetButtonsForRaize;
  end
end;

procedure TFormDrawPoker.HemBtnFoldClick(Sender: TObject);
begin
  if MessageDlg('Are you sure you want to fold', mtConfirmation, [mbYes, mbNo], 0)
  = mrYes then
  begin
    LblComputerBet.Caption := '';
    DisableAllButtons;
    EnableButton(HemBtnDeal);
  end;
end;

procedure TFormDrawPoker.HemBtn10Click(Sender: TObject);
var
  i : Integer;
begin
  for i := 1 to 5 do
    if Sender = BetValueButtons[i] then
      BetValueButtons[i].FaceColor := clRed
    else
      BetValueButtons[i].FaceColor := clYellow;
  SelectedValue := StrToInt((Sender as THemisphereButton).Caption);
  SelectedButton := THemisphereButton(Sender);
end;

procedure TFormDrawPoker.Card1Click(Sender: TObject);
begin
  if HemBtnDraw.Enabled then
  begin
    if Sender = Card5 then
      LblHeld5.Visible := not LblHeld5.Visible
    else
      if Sender = Card4 then
        LblHeld4.Visible := not LblHeld4.Visible
      else
        if Sender = Card3 then
          LblHeld3.Visible := not LblHeld3.Visible
        else
          if Sender = Card2 then
            LblHeld2.Visible := not LblHeld2.Visible
          else
            if Sender = Card1 then
              LblHeld1.Visible := not LblHeld1.Visible;
  end;
end;

procedure TFormDrawPoker.DealTimerTimer(Sender: TObject);
{Deal to players alternately at speed set by timer Interval}
begin
  if CardIndex <= 10 then
  begin
    CardComponents[CardIndex].Show;
    case ComputerBetsFirst of
      True: if (CardIndex >= 6) then
              Dec(CardIndex,5)
            else
              Inc( CardIndex,6);
      False: if (CardIndex < 6) or (CardIndex = 10) then
              Inc(CardIndex,5)
             else
               Dec(CardIndex,4);
    end
  end else             // Cards are now dealt
  begin
    case ComputerBetsFirst of
      True:  begin                // Computer's bet
               ComputerPlayer.BetOrPass (UserPlayer);
               if ComputerPlayer.BetValue > 10 then
                 SetBetValueButtons;    // Computer's bet sets the new minimum bet
               DisplayBet;
               AddBetToPot (ComputerPlayer);
               if ComputerPlayer.Passed then
                 SetButtonsForBet
               else
                 SetButtonsForRaize;
             end;
      False: SetButtonsForBet;     // User's bet
    end;
    DealTimer.Enabled := False;    // Disable timer when deal done
  end;
end;

procedure TFormDrawPoker.ComputerDrawTimerTimer(Sender: TObject);
{Change cards at speed set by timer Interval}
var
  S : string;
begin
  if CardIndex <= 10 then
  begin
    if not CardComponents[CardIndex].visible then
      CardComponents[CardIndex].Show;
    Inc(CardIndex);
  end else
  begin
    S := IntToStr(ComputerPlayer.NumberDrawn);
    case ComputerPlayer.NumberDrawn of
      1:  S := '   Computer has changed '+ s + ' card.';
      else S := '   Computer has changed '+ s + ' cards.';
    end;
    if not ComputerBetsFirst then
      S:= S + ''
    else
      S:= S + '   Click on cards you want to keep';
    Caption := S;
    ComputerPlayer.SortHand(1,5);
    ComputerPlayer.SetRank;
    SetCards(ComputerPlayer);
    ComputerDrawTimer.Enabled := False;   // Disable timer when draw done
  end;
end;

procedure TFormDrawPoker.UserDrawTimerTimer(Sender: TObject);
{Change cards at speed set by timer Interval}
begin
  if CardIndex <= 5 then
  begin
    if not CardComponents[CardIndex].Visible then
      CardComponents[CardIndex].Show;
    Inc(CardIndex);
  end
  else begin
    UserPlayer.SortHand(1,5);
    SetCards(UserPlayer);
    UserPlayer.SetRank;
    UserDrawTimer.Enabled := False;   // Disable timer when draw done
    if not ComputerBetsFirst then
      ComputerDraw;
  end;
end;

procedure TFormDrawPoker.FormDestroy(Sender: TObject);
begin
  Pack.Free;
  ComputerPlayer.Free;
  UserPlayer.Free;
end;

end.


