unit PokerPlayer;

interface
uses
  Pack;
const
   Ante = 10;
type
  TRank = (rkNothing, rkPair, rkTwoPair, rkThrees, rkStraight, rkFlush, rkFullHouse,
    rkFours, rkStraightFlush);
  TUnRanked = (urkNoHighCard, urkOneHighCard, urkTwoHighCards, urkFourFlush,
    urkFourStraight, urkFourStraightFlush);
  TNumberDrawn = 0..5;

  THand = record
    Cards: T5CardArray;
    DrawCards: T5CardArray;
    Discard: Integer;
    SubRank: TValue;
    case Rank: TRank of
      rkNothing, rkPair: (UnRanked : TUnRanked);
      rkTwoPair : (SubSubRank: TValue);
    end;

  TPokerPlayer = class       // base class
  private
    FHand: THand;
    FBetValue: Integer;
    FRaizeValue: Integer;
    FNumberDrawn: TNumberDrawn;
    FRaized: Boolean;
    FFolded: Boolean;
    FCalled: Boolean;
    FPassed: Boolean;
    FPairPosition: Integer;
    FThreesPosition: Integer;
    FFoursPosition: Integer;
  public
    procedure InitNewDeal(const APack: TPack); virtual;
    procedure SetRank;
    procedure SortHand(Lo, Hi: Integer);
    function CardValue(Position: Integer) : TValue;
    function SuitValue(Position: Integer) : TSuit;
    procedure DrawCard(Position: Integer);
    property Discard: Integer read FHand.Discard;
    property PairPosition: Integer read FPairPosition;
    property ThreesPosition: Integer read FThreesPosition;
    property FoursPosition: Integer read FFoursPosition;
    property Rank: TRank read FHand.Rank;
    property SubRank: TValue read FHand.SubRank;
    property SubSubRank: TValue read FHand.SubSubRank;
    property UnRanked: TUnRanked read FHand.UnRanked;
    property BetValue: Integer read FBetValue write FBetValue;
    property RaizeValue: Integer read FRaizeValue write FRaizeValue;
    property NumberDrawn: TNumberDrawn read FNumberDrawn write FNumberDrawn;
    property Raized: Boolean read FRaized write FRaized;
    property Folded: Boolean read FFolded write FFolded;
    property Called: Boolean read FCalled write FCalled;
    property Passed: Boolean read FPassed write FPassed;
  end;

implementation

procedure TPokerPlayer.DrawCard(Position: Integer);
{Change card in place Position}
begin
  FHand.Cards[Position] := FHand.DrawCards[Position];
  Inc(FNumberDrawn);
end;

function TPokerPlayer.CardValue(Position: Integer) : TValue;
begin
  Result := FHand.Cards[Position].Value;
end;

function TPokerPlayer.SuitValue(Position: Integer) : TSuit;
begin
  Result := FHand.Cards[Position].Suit;
end;

procedure TPokerPlayer.InitNewDeal(const APack: TPack);
begin
  FHand.Cards := APack.Draw5Cards;
  FHand.DrawCards := APack.Draw5Cards; // Set cards available for replacing discards
  FHand.Discard := 1;
  FHand.Rank := rkNothing;
  FHand.SubRank := vlTwo;
  FHand.SubSubRank := vlTwo;
  FHand.UnRanked := urkNoHighCard;
  FBetValue := 0;
  FRaizeValue := 0;
  FNumberDrawn:=0;
  FFolded := False;
  FCalled := False;
  FPassed := False;
  FRaized := False;
  FPairPosition := 5;
  FThreesPosition := 5;
  FFoursPosition := 5;
  SortHand(1,5);
  SetRank;
end;

procedure TPokerPlayer.SortHand(Lo, Hi: Integer);
{Sort with highest to the right}
  procedure Srt(Left, Right: Integer);
  var
   i, j, x: Integer;
   Temp: TPokerCard;
  begin
    i := Left; j := Right;
    x := Ord(FHand.Cards[(Left+Right) div 2].Value);
    repeat
      while Ord(FHand.Cards[i].Value) < x do
        i := i + 1;
      while x < Ord(FHand.cards[j].Value) do
        j := j - 1;
      if i <= j then
      begin
        Temp := FHand.Cards[i];
        FHand.Cards[i] := FHand.Cards[j];
        FHand.Cards[j] := Temp;
        i := i + 1;
        j := j - 1;
      end;
    until i > j;
    if Left < j then
      SortHand(Left, j);
    if i < Right then
      SortHand(i, Right);
  end;
begin
  Srt(Lo,Hi);
end;

procedure TPokerPlayer.SetRank;
  procedure SearchForRuns(var Found: Boolean; Start, Finish: Integer);
  begin
    Found:= True;
    while (Found) and (Start > Finish) do
      begin
        if Ord(CardValue(Start)) - Ord(CardValue(Start-1)) <> 1 then
          Found := False;
        Dec(Start);
      end;
  end;
  procedure SearchForCollections(var Found: Boolean; var Position: Integer;
    HowMany: Integer);
  var
    i: Integer;
  begin
    i := 5;
    Found := False;
    while (not Found) and (i >= HowMany) do
    begin
      case HowMany of
        2:  if CardValue(i) = CardValue(i-1) then
              Found := True
            else
              Dec(i);
        3:  if (CardValue(i) = CardValue(i-1)) and (CardValue(i) = CardValue(i-2)) then
              Found := True
            else
              Dec(i);
        4:  if (CardValue(i) = CardValue(i-1)) and (CardValue(i) = CardValue(i-2))
              and (CardValue(i) = CardValue(i-3)) then
                Found := True
              else
                Dec(i);
      end;
    end;
    Position := i;
    if Found then
      FHand.SubRank := CardValue(i);
  end;
  function FourStraightFound: Boolean;
  { Open-ended straight}
  var
    Found: Boolean;
  begin
    Found := True;
    if Ord(CardValue(5)) - Ord(CardValue(4)) > 1 then
    begin
      FHand.Discard := 5;
      SearchForRuns(Found,4,1);
    end else
      if CardValue(5) = vlAce then
        Found := False
      else begin
         FHand.Discard := 1;
         SearchForRuns(Found,5,2);
      end;
    Result := Found;
  end;

  function FlushFound: Boolean;
  var
    i : Integer;
    Found : Boolean;
  begin
    Found := True;
    i := 5;
    while (Found) and (i > 1) do
    begin
      if SuitValue(i) <> SuitValue(i-1) then
        Found := False;
      Dec(i);
    end;
    Result := Found;
  end;

  function StraightFound: Boolean;
  var
    Found: Boolean;
  begin
    SearchForRuns(Found,5,1);
    if (not Found) and (CardValue(5) = vlAce) and (CardValue(4) = vlFive) then
      SearchForRuns(Found,3,1);
    Result := Found;
  end;

  function FourFlushFound: Boolean;
  var
    SpadeCount, DiamondCount, ClubCount, HeartCount, i  : Integer;
    Found : Boolean;
  begin
    Found := False;
    SpadeCount:= 0;
    DiamondCount:= 0;
    ClubCount:= 0;
    HeartCount:=0;
    for i:= 1 to 5 do
    begin
      if SuitValue(i) = stSpade then Inc(SpadeCount);
      if SuitValue(i) = stDiamond then Inc(DiamondCount);
      if SuitValue(i) = stClub then Inc(ClubCount);
      if SuitValue(i) = stHeart then inc(HeartCount);
    end;
    if (SpadeCount = 4) or (DiamondCount = 4) or (ClubCount = 4) or
      (HeartCount = 4)then
        Found := True;
    Result := Found;
    if Found then
    begin
      if (SuitValue(5) = SuitValue(4)) and (SuitValue(3) = SuitValue(2)) then
        FHand.Discard := 1
      else
        if (SuitValue(5) = SuitValue(4)) and (SuitValue(3) = SuitValue(1)) then
          FHand.Discard := 2
        else
          if (SuitValue(5) = SuitValue(1)) and (SuitValue(3) = SuitValue(2)) then
            FHand.Discard := 4
          else
            if (SuitValue(5) = SuitValue(4)) and
             (SuitValue(1) = SuitValue(2)) then
               FHand.Discard := 3
               else
                if (SuitValue(1) = SuitValue(4)) and
                  (SuitValue(3) = SuitValue(2)) then
                     FHand.Discard := 5;
     end;
   end;

   function FourStraightFlushFound: Boolean;
   var
     Found: Boolean;
   begin
     Found := False;
     if FourFlushFound then
     begin
       if (FHand.Discard = 5) or (CardValue(5) = vlAce) then
         SearchForRuns(Found,4,1)
       else
         if FHand.Discard = 1 then
           SearchForRuns(Found,5,2);
       end;
     Result := found;
   end;

  function PairFound: Boolean;
  var
    Found : Boolean;
  begin
    SearchForCollections(Found,FPairPosition,2);
    Result := Found;
  end;

  function TwoPairsFound : Boolean;
  {Test for another pair if pair already found}
  var
    Found : Boolean;
  begin
    Found := False;
    case FPairPosition of        // Must be 4 or 5 if 2 pairs exist
      5: if (CardValue(2) = CardValue(1)) or (CardValue(2) = CardValue(3))then
           Found := True;
      4: if CardValue(2) = CardValue(1) then
           Found := True;
    end;
    if Found then
    begin
      FHand.SubSubRank := CardValue(2);
      if CardValue(2) = CardValue(1) then
        FHand.Discard := 3
      else
        FHand.Discard := 1;
    end;
    Result := Found;
  end;

  function ThreesFound : Boolean;
  var
    Found : Boolean;
  begin
      SearchForCollections(Found,FThreesPosition,3);
      Result := Found;
  end;

  function FoursFound : Boolean;
  var
    Found : Boolean;
  begin
    SearchForCollections(Found,FFoursPosition,4);
    Result := Found;
  end;

begin
  FHand.Rank := rkNothing;
  FPairPosition := 0;
  if PairFound then
  begin
    FHand.Rank := rkPair;                             // PAIR
    if FourStraightFlushFound then
      FHand.UnRanked := urkFourStraightFlush          // FOUR STRAIGHT FLUSH
    else begin
      if TwoPairsFound then
        FHand.Rank := rkTwoPair;                      // TWO PAIRS
      if ThreesFound then
        FHand.Rank := rkThrees;                       // THREES
      if TwoPairsFound and ThreesFound then
        FHand.Rank := rkFullHouse;                    // FULL HOUSE
      if FoursFound then
        FHand.Rank := rkFours;                        // FOURS
    end
  end else
    begin  // Pair not found
      if FlushFound then
        FHand.Rank:= rkFlush;                           // FLUSH
      if StraightFound then
        FHand.Rank:= rkStraight;                        // STRAIGHT
      if (StraightFound) and (FlushFound) then
        FHand.Rank:= rkStraightFlush;                   // STRAIGHT FLUSH
      if FHand.Rank = rkNothing then
        if FourStraightFlushFound then
          FHand.UnRanked := urkFourStraightFlush
        else
          if fourFlushFound then
            FHand.UnRanked := urkFourFlush
          else
            if fourStraightFound then
              FHand.UnRanked := urkFourStraight
            else begin
              case CardValue(4) of
                vlJack..vlKing:  FHand.UnRanked := urkTwoHighCards;
                vlSix..vlTen: if CardValue(5) in [vlJack..vlAce] then
                                FHand.UnRanked := urkOneHighCard
                              else
                                FHand.UnRanked := urkNoHighCard;
                else FHand.UnRanked := urkNoHighCard;
              end;
    end;
  end;
end;

end.
